<?php

/**
 * Class MySQLi
 *
 * LICENSE: http://opensource.org/licenses/gpl-license.php GNU Public License
 *
 * @version    1.0
 * @author Кот ДаWINчи
 * 
 * Данный класс предназначен для работы с базой данных MySQL через 
 * модуль MySQLi. Он для создан для упращения и стандартизации всех 
 * запросов к базе данных и более простого перехода к другой системе 
 * хранения информации.
 */
class DBClass
    {

    private $host = '127.0.0.1';       // адрес подключения
    private $user = 'root';            // имя пользователя
    private $password = '12345';       // пароль
    private $port = '3306';            // порт
    private $db = '';                  // имя базы данных
    private $charset = 'utf8';         // кодировка БД
    public $error = false;             // Флаг ошибки
    public $error_text = '';           // Текст ошибки
    public $lastQuery = '';            // Текст последнего запроса
    private $rows = 0;                  // количество выбранных записей
    public $fields = 0;                // количество выбранных полей(столбцов)
    public $fields_name = array();     // Массив выбранных полей(столбцов) в таблице
    public $tables_name = array();     // Массив выбранных таблиц в базе
    public $databases_name = array();  // Массив выбранных баз данных
    private $result = null;            // Массив результата
    private $mysqli;                   // внутренняя переменная поделючения к MySQLi

    /**
     * Деструктор класса
     */

    private function __destruct()
        {
        $this->mysqli->close();
        }

    /**
     * * Конструктор класса
     * 
     * @param string $chost адрес/имя сервера
     * @param string $cuser логин к серверу БД
     * @param string $cpass пароль
     * @param string $cdb имя базы
     * @param string $cport порт сервера
     * @param string $ccharset кодировка (поумолчанию utf8)
     * @return boolean
     */
    private function __construct($chost = '', $cuser = '', $cpass = '', $cdb = '', $cport = '', $ccharset = '')
        {
        // Если введены данные для подключения, то сохраняем их в переменных класса.
        // иначе используем данные "по-умолчанию".
        if ($chost <> '')
            $this->host = $chost;
        if ($cuser <> '')
            $this->user = $cuser;
        if ($cpass <> '')
            $this->password = $cpass;
        if ($cport <> '')
            $this->port = $cport;
        if ($cdb <> '')
            $this->db = $cdb;
        if ($ccharset <> '')
            $this->charset = $ccharset;
        // Сброс флага ошибки
        $this->errorClear();
        // Попытка подключения
        $this->mysqli = new mysqli($this->host, $this->user, $this->password, $this->db, $this->port);
        // проверка на ошибку подключения
        if ($this->mysqli->connect_error)
            {
            $this->error = true;
            $this->error_text = $this->mysqli->connect_error;
            } else
            {
            // если не было ошибки, то устанавливаем кодировку
            $this->mysqli->set_charset($this->charset);
            }
        // Возвращаем ИСТИНА при подключении или ЛОЖЬ при ошибке.
        return!$this->error;
        }

    /**
     * Сброс флага ошибки.
     * @return void
     */
    private function errorClear()
        {
        $this->error = false; // Сброс флага ошибки
        $this->error_text = ''; // Сброс текста
        }

    /**
     * функция возвращает имя текущей БД.
     * 
     * @return string
     */
    public function getDbName()
        {
        return $this->db;
        }

    /**
     * функция фозвращает количество выбранных записей последнего запроса.
     * @return int
     */
    public function countRows()
        {
        return $this->rows;
        }

    /**
     * функция фозвращает количество выбранных столбцов(полей) последнего запроса.
     * @return int
     */
    public function countFields()
        {
        return $this->fields;
        }

    /**
     * Универсальный запрос. Подготовленное тело запроса передается через 
     * параметр или берется из переменной $this->lastQuery.
     * @param string $CurrentQuery Строка SQL запроса
     * @return boolean
     */
    public function query($CurrentQuery)
        {
        // Сброс флага ошибки
        $this->errorClear();
        // если есть параметр, то занести его в переменную запроса.
        if ($CurrentQuery)
            {
            $this->lastQuery = $CurrentQuery;
            }
        // Подготовка запроса
        $query = $this->mysqli->prepare($this->lastQuery);
        // исполнение
        if ($query->execute())
            {
            // сохранение результата
            $this->result = $query->get_result();
            $query->store_result();
            // сохраняем количество выбранных записей.
            $this->rows = $query->num_rows;
            // сохраняемых количество полученных столбцов(полей) для каждой записи.
            $this->fields = $query->field_count;
            // освобождаем память
            $query->free_result();
            } else
            {
            // если была ошибка, то фиксируем это
            $this->error = true;
            $this->error_text = $query->error;
            }
        $query->close();
        //  возвращаем флаг Успеха.
        return!($this->error);
        }

    /**
     *  Запрос на определение количества вида: SELECT COUNT(*) FROM table WHERE ...
     *  Возвращает количество отобранных записей или 0 (даже при ошибке).
     *  Если была ошибка, то флаг и текст ошибки заполняется в переменные класса.
     */
    public function queryCount($table, $where = '')
        {
        $count = 0;
        // Сброс флага ошибки
        $this->errorClear();
        // формируем строку запроса
        $this->lastQuery = 'SELECT COUNT(*) FROM `' . $table . '`';
        // если условие не пусто, то добавляем его в запрос.
        if ($where <> '')
            {
            $this->lastQuery .= ' WHERE ' . $where;
            }
        // подготовка
        $query = $this->mysqli->prepare($this->lastQuery);
        // выполнение запроса
        if ($query->execute())
            {
            $query->bind_result($count);
            $query->fetch();
            $query->free_result();
            } else
            {
            // если была ошибка, то фиксируем это
            $this->error = true;
            $this->error_text = $query->error;
            }
        $query->close();
        return $count;
        }

    /**
     *  получить именнованую строку из результата запроса.
     */
    public function fetchAssoc()
        {
        return $this->result->fetch_assoc();
        }

    /**
     *  получить НЕименнованую строку из результата запроса.
     */
    public function fetchArray()
        {
        return $this->result->fetch_array();
        }

    /**
     * Формирование перечня таблиц в базе
     */
    public function showTables($table = '')
        {
        // Сброс флага ошибки
        $this->errorClear();
        // формируем строку запроса
        $this->lastQuery = 'SHOW TABLES';
        // если условие не пусто, то добавляем его в запрос
        if ($table <> '')
            {
            $this->lastQuery .= ' LIKE "' . $table . '"';
            }
        // подготовка
        $count = 0;
        $query = $this->mysqli->prepare($this->lastQuery);
        // выполнение запроса
        if ($query->execute())
            {
            $show_result = $query->get_result();
            $query->store_result();
            $query->free_result();
            // опустошаем массив
            $this->tables_name = array();
            // заполняем массив данными из запрса
            while ($name = $show_result->fetch_array())
                {
                $this->tables_name[] = $name['0'];
                $count++; // подсчет количества таблиц
                }
            } else
            {
            // если была ошибка, то фиксируем это
            $this->error = true;
            $this->error_text = $query->error;
            }
        $query->close();
        return $count;
        }

    /**
     * Формирование перечня полей в таблице 
     */
    public function showFields($table, $mask = '')
        {
        // Сброс флага ошибки
        $this->errorClear();
        // формируем строку запроса
        $this->lastQuery = 'SHOW COLUMNS FROM `' . $table . '`';
        // если условие не пусто, то добавляем его в запрос
        if ($mask <> '')
            {
            $this->lastQuery .= ' WHERE `field` LIKE "' . $mask . '"';
            }
        // подготовка
        $count = 0;
        $query = $this->mysqli->prepare($this->lastQuery);
        // выполнение запроса
        if ($query->execute())
            {
            $show_result = $query->get_result();
            $query->store_result();
            $query->free_result();
            // опустошаем массив
            $this->fields_name = array();
            // заполняем массив данными из запрса
            while ($name = $show_result->fetch_array())
                {
                $this->fields_name[] = $name['0'];
                $count++;
                }
            } else
            {
            // если была ошибка, то фиксируем это
            $this->error = true;
            $this->error_text = $query->error;
            }
        $query->close();
        return $count;
        }

    /**
     * Формирование баз данных 
     */
    public function showDbs($mask = '')
        {
        // Сброс флага ошибки
        $this->errorClear();
        // формируем строку запроса
        $this->lastQuery = 'SHOW DATABASES';
        // если условие не пусто, то добавляем его в запрос
        if ($mask <> '')
            {
            $this->lastQuery .= ' LIKE "' . $mask . '"';
            }
        // подготовка
        $count = 0;
        $query = $this->mysqli->prepare($this->lastQuery);
        // выполнение запроса
        if ($query->execute())
            {
            $show_result = $query->get_result();
            $query->store_result();
            $query->free_result();
            // опустошаем массив
            $this->databases_name = array();
            // заполняем массив данными из запрса
            while ($name = $show_result->fetch_array())
                {
                $this->databases_name[] = $name['0'];
                $count++;
                }
            } else
            {
            // если была ошибка, то фиксируем это
            $this->error = true;
            $this->error_text = $query->error;
            }
        $query->close();
        return $count;
        }

    /**
     * Смена базы данных.
     */
    public function selectDb($new_db, $newcharset = '')
        {
        //Пробуем сменить базу данных
        if ($this->mysqli->select_db($new_db))
            {
            // если была задана кодировка базы, то изменяем ее в классе
            if ($newcharset <> '')
                {
                $this->charset = $newcharset;
                }
            $this->mysqli->set_charset($this->charset);
            // если успешно, то правим переменные класса
            $this->db = $new_db;
            $this->errorClear();
            } else
            {
            // если не смогли, то выставляем флаг и текст ошибки
            $this->error = true;
            $this->error_text = $this->mysqli->error;
            // и переключаемся на старую базу
            $this->mysqli->select_db($this->db);
            $this->mysqli->set_charset($this->charset);
            }
        // возвращаем истину если сменили, и ложь если ошибка.
        return!$this->error;
        }

    }

?>